import json
import webbrowser
import httplib2
from googleapiclient.discovery import build
from oauth2client import client
import os
from builtins import input

def saveJson(filename, object):
    with open(filename, 'w') as f:
        json.dump(object, f)


def openJson(filename):
    with open(filename, 'r') as f:
        object = json.load(f)
    return object


def getCredentials(secrets, credentials, scopes):
    if not os.path.isfile(credentials):
        flow = client.flow_from_clientsecrets(
                secrets,
                scope=scopes,
                redirect_uri='urn:ietf:wg:oauth:2.0:oob')
        auth_uri = flow.step1_get_authorize_url()
        print("Auth url: {}".format(auth_uri))
        webbrowser.open(auth_uri)
        auth_code = input('Enter the auth code: ')
        cre = flow.step2_exchange(auth_code)
        saveJson(credentials,cre.to_json())
    else:
        cre = client.Credentials.new_from_json(openJson(credentials))
    return cre


def create_api(api_name, api_version, scopes=None, secrets=None, credentials=None):
    if None in (secrets, credentials, scopes):
        return build(api_name, api_version)
    # else:
    #     raise ValueError("The variables {}, {} and {} should not be empty if there is no SA available!".format(scopes, secrets, credentials))
    http_auth = getCredentials(secrets, credentials, scopes).authorize(httplib2.Http())
    return build(api_name, api_version, http=http_auth)



"""Analytics Manager module.

This module have a class with built in functionality for Google Analytics.
"""
__author__ = 'Metriplica-Ayyoub'

import time
import os
import json
import re
import hashlib
import logging
import shutil
import uuid
import pandas as pd
from io import BytesIO
from pykemen.utilities import create_api
from datetime import datetime, timedelta
from googleapiclient.http import MediaFileUpload, HttpError, MediaIoBaseUpload

logger = logging.getLogger("Analytics")
logger.setLevel(logging.WARNING)

class Analytics(object):
    """Google Analytics class.

    Query unsampled reports from Analytics and cache results.
    Also has a built in method to upload data to Analytics through data import."""
    CACHE_DIR = './cache/{profile}/{id}/'
    CACHE_REPORT = './cache/{profile}/{id}/report_{start_date}_{end_date}.csv'
    CACHE_UNSAMPLED_REPORT = './cache/{profile}/{id}/unsampled_report_{date}.csv'

    class AnalyticsReport(object):
        """"AnalyticsReport class.

        Stores all properties of an Analytics report and returns a dataFrame of the report."""
        REPORT_RE = r"report_[0-9]{4}-[0-9]{2}-[0-9]{2}_[0-9]{4}-[0-9]{2}-[0-9]{2}\.csv"
        UNSAMPLED_REPORT_RE = r'unsampled_report_[0-9]{4}-[0-9]{2}-[0-9]{2}\.csv'

        def __init__(self, path, start_date, end_date, dimensions, metrics, filters, segments, sort, df, unsampled=False, cache=True):
            """Init method initialize and create AnalyticsReport class.

            Args:
                path (str): path where report files are stored.
                start_date (str): start date of the report (format: %Y-%m-%d)
                end_date (str): end date of the report (format: %Y-%m-%d)
                dimensions (str): comma separated Analytics dimensions
                metrics (str): comma separated Analytics metrics
                filters (str): filters used with analytics api
                segments (str): segments used with analytics api
                sort (str): comma separated dimensions and metrics to sort by the report.

            Returns:
                Analytics.AnalyticsReport: with the given configuration.
            """
            self.path = path
            self.report_re = Analytics.AnalyticsReport.UNSAMPLED_REPORT_RE if unsampled else Analytics.AnalyticsReport.REPORT_RE
            self.start_date = start_date
            self.end_date = end_date
            self.dimensions = dimensions.split(",") if dimensions else None
            self.metrics = metrics.split(",")
            self.filters = filters
            self.segments = segments
            self.sort = sort.split(",")
            self.unsampled = unsampled
            self.cache = cache
            self._df=df

        def to_data_frame(self):
            """Retrieve report into a pandas dataFrame.

            Reads file reports from cache and groups all the required files into a single dataFrame.

            Returns:
                pd.DataFrame
            """
            if not self.cache:
                return self._df
            filenames = os.listdir(self.path)
            filenames = list(filter(lambda x: re.match(self.report_re, x), filenames))
            if self.unsampled:
                filenames = list(filter(lambda x: filter_report_files_by_date(x, self.start_date, self.end_date), filenames))
            else:
                filenames = list(filter(lambda x: x == 'report_{start_date}_{end_date}.csv'.format(
                    start_date=self.start_date, end_date=self.end_date), filenames))
            
            if not len(filenames):
                columns = self.dimensions
                columns.extend(self.metrics)
                return pd.DataFrame(columns=columns)
            filenames.sort()
            dataframes = (pd.read_csv(self.path + filename, index_col=False, dtype=self._get_dtypes()) for filename in
                        filenames)
            dataframe = pd.concat(dataframes, ignore_index=True)
            dataframe = dataframe.groupby(self.dimensions).sum().reset_index()
            for sort in self.sort:
                if sort.startswith('-'):
                    dataframe = dataframe.sort_values(by=[sort[1:]], ascending=False)
                elif sort != '':
                    dataframe = dataframe.sort_values(by=[sort])
            return dataframe

        def to_csv(self, filename):
            """Stores the report into a csv file.

            Args:
                filename (str): path of the filename to store the report into.
            """
            df = self.to_data_frame()
            df.to_csv(filename, encoding='utf-8', index=False)

        def _get_dtypes(self):
            """Returns the types of the report to format correctly the pd.DataFrame.

            Returns:
                dict
            """
            dtypes = {dimension: str for dimension in self.dimensions}
            dtypes.update({metric: float for metric in self.metrics})
            return dtypes

    def __init__(self, credentials=None, secrets=None):
        """Constructor for Analytics class.

        Args:
            credentials(str): json filename with oauth credentials.
            secrets(str): json filename with the access, if there is non, one will be created.

        Returns:
            Analytics
        """
        super(Analytics, self).__init__()
        scope = [
            "https://www.googleapis.com/auth/analytics.edit", 
            "https://www.googleapis.com/auth/analytics",
            "https://www.googleapis.com/auth/analytics.manage.users",
            ]
        self._analyticsService = create_api("analytics", "v3", scope, secrets, credentials)
        self._uuid = uuid.uuid4()

    def get_report(self, unsampled=False, cache=True, **kwargs):
        """Downloads data from Analytics and caches the result. If the data is alredy cached, skips the
        download and directly returns an Analytics.AnalyticsReport.

        Args:
            unsampled (boolean): True will download the report day by day to try get unsampled data.
            kwargs (**dict): Analytics report configuration variable with all required parameters

        Returns:
            Analytics.AnalyticsReport
        """
        
        id_to_hash = ",".join([
            kwargs.get("dimensions", ""), 
            kwargs.get("metrics", ""),
            kwargs.get("filters", ""),
            kwargs.get("segments", ""),
            ])
        kwargs["quotaUser"] = self._uuid
        id_ = hashlib.md5(id_to_hash.encode('utf8')).hexdigest()
        start_date = kwargs.get('start_date')
        end_date = kwargs.get('end_date')
        columns = ','.join([kwargs.get('dimensions'), kwargs.get('metrics')])
        columns = columns.split(',')
        dtypes = {dimension: str for dimension in kwargs.get("dimensions","").split(",")}
        dtypes.update({metric: float for metric in kwargs.get("metrics", "").split(",")})
        rows = []
        data_frames = []
        report = None
        if cache and not os.path.isdir(Analytics.CACHE_DIR.format(profile=kwargs.get('ids').replace('ga:', ''), id=id_)):
            os.makedirs(Analytics.CACHE_DIR.format(profile=kwargs.get('ids').replace('ga:', ''), id=id_))
        
        if not unsampled:
            filename = Analytics.CACHE_REPORT.format(
                profile=kwargs.get('ids').replace('ga:', ''), 
                id=id_, 
                start_date=start_date, 
                end_date=end_date
            )
            if not cache or not self._in_cache(kwargs.get('ids').replace('ga:', ''), id_, start_date, end_date):
                qps = True
                while qps:
                    qps = False
                    try:
                        report = self._analyticsService.data().ga().get(**kwargs).execute()
                    except HttpError as e:
                        error_content = json.loads(e.content.decode("utf-8"))
                        if error_content.get("error", {}).get("message") =="Rate Limit Exceeded":
                            qps = True
                            time.sleep(1)
                            logger.warn("Rate limit excedeed!")
                            logger.warn(e.content)
                        else:
                            logger.warn(e.content)
                            raise e
                if report.get('containSampledData'):
                    logger.warn("There are sampled results on the report: {dimensions}{metrics} - date: {start_date} to {end_date}".format(
                            dimensions=kwargs.get("dimensions"), metrics=kwargs.get("metrics"), start_date=start_date, end_date=end_date))
                rows.extend(report.get('rows', []))
                iteration = 1
                while report.get('nextLink'):
                    kwargs['start_index'] = 1 + kwargs.get('max_results', 1000) * iteration
                    qps = True
                    while qps:
                        qps = False
                        try:
                            report = self._analyticsService.data().ga().get(**kwargs).execute()
                        except HttpError as e:
                            error_content = json.loads(e.content.decode("utf-8"))
                            if error_content.get("error", {}).get("message") =="Rate Limit Exceeded":
                                qps = True
                                time.sleep(1)
                                logger.warn("Rate limit excedeed!")
                                logger.warn(e.content)
                            else:
                                logger.warn(e.content)
                                raise e
                    rows.extend(report.get('rows', []))
                    iteration += 1
                df = pd.DataFrame(data=rows, columns=columns)

                if cache:
                    df.to_csv(filename, index=False, encoding='utf-8')
                    logger.info('Saved file' + filename)
            else:
                df = pd.read_csv(filename, index_col=False, dtype=dtypes)
        else:
            startDate = datetime.strptime(start_date, "%Y-%m-%d")
            endDate = datetime.strptime(end_date, "%Y-%m-%d")
            diffDays = (endDate - startDate).days + 1
            for day in range(diffDays):
                actualDate = (startDate + timedelta(days=day)).strftime("%Y-%m-%d")
                filename = Analytics.CACHE_UNSAMPLED_REPORT.format(profile=kwargs.get('ids').replace('ga:', ''), id=id_,
                                                        date=actualDate)
                if  not cache or not self._in_cache_by_day(kwargs.get('ids').replace('ga:', ''), id_, actualDate):
                    kwargs["start_date"] = actualDate
                    kwargs["end_date"] = actualDate
                    kwargs["start_index"] = 1
                    qps = True
                    while qps:
                        qps = False
                        try:
                            report = self._analyticsService.data().ga().get(**kwargs).execute()
                        except HttpError as e:
                            error_content = json.loads(e.content.decode("utf-8"))
                            if error_content.get("error", {}).get("message") =="Rate Limit Exceeded":
                                qps = True
                                time.sleep(1)
                                logger.warn("Rate limit excedeed!")
                                logger.warn(e.content)
                            else:
                                logger.warn(e.content)
                                raise e
                    if report.get("containsSampledData"):
                        logger.warn("There are sampled results on the report: {dimensions}{metrics} - date{date}".format(
                            dimensions=kwargs.get("dimensions"), metrics=kwargs.get("metrics"), date=actualDate))
                    rows.extend(report.get("rows", []))
                    iteration = 1
                    while report.get("nextLink"):
                        kwargs["start_index"] = 1 + kwargs.get('max_results', 1000) * iteration
                        qps = True
                        while qps:
                            qps = False
                            try:
                                report = self._analyticsService.data().ga().get(**kwargs).execute()
                            except HttpError as e:
                                error_content = json.loads(e.content.decode("utf-8"))
                                if error_content.get("error", {}).get("message") =="Rate Limit Exceeded":
                                    qps = True
                                    time.sleep(1)
                                    logger.warn("Rate limit excedeed!")
                                    logger.warn(e.content)
                                else:
                                    logger.warn(e.content)
                                    raise e
                        rows.extend(report.get("rows", []))
                        iteration += 1
                    df = pd.DataFrame(data=rows, columns=columns)
                    data_frames.append(df)
                    if cache:
                        df.to_csv(filename, index=False, encoding='utf-8')
                        logger.info("Saved file " + filename)
                    rows = []
                else:
                    df = pd.read_csv(filename, index_col=False, dtype=dtypes)
                    data_frames.append(df)
                
            df = pd.concat(data_frames, ignore_index=True)
            df = df.astype(dtypes)
            df = df.groupby(kwargs.get("dimensions", "").split(",")).sum().reset_index()
            for sort in kwargs.get("sort", "").split(","):
                if sort.startswith('-'):
                    df = df.sort_values(by=[sort[1:]], ascending=False)
                elif sort != '':
                    df = df.sort_values(by=[sort])
        df = df.astype(dtypes)
        return Analytics.AnalyticsReport(
            Analytics.CACHE_DIR.format(profile=kwargs.get('ids', '').replace('ga:', ''), id=id_),
            start_date,
            end_date,
            kwargs.get('dimensions', ''),
            kwargs.get('metrics', ''),
            kwargs.get('filters', ''),
            kwargs.get('segments', ''),
            kwargs.get('sort', ""),
            df,
            unsampled=unsampled,
            cache=cache
        )

    def data_import_upload_data(self, accountId, webPropertyId, dataSourceId, filename=None, content=None):
        """Import a csv to Analytics through a data import.

        Args:
            accountId (str): Analytics account id to upload the data to
            webPropertyId (str): Property Id where the data import targeted is
            dataSourceId (str): Id of the data import to upload de data to
            filename (str): file path to upload to Analytics

        Returns:
            None if the upload succeed, raise an error otherwise"""
        if filename is None and content is None:
            raise Exception("In order to upload data, you have either to introduce a valid filename or a content.")
        if filename:
            media = MediaFileUpload(filename, mimetype='application/octet-stream', resumable=False)
        else:
            media = MediaIoBaseUpload(BytesIO(content.encode("utf-8")), mimetype='application/octet-stream', resumable=False)
        response = self._analyticsService.management().uploads().uploadData(
            accountId=accountId,
            webPropertyId=webPropertyId,
            customDataSourceId=dataSourceId,
            media_body=media).execute()
        while response.get('status') == 'PENDING':
            time.sleep(60)
            response = self._analyticsService.management().uploads().get(
                accountId=accountId,
                webPropertyId=webPropertyId,
                customDataSourceId=dataSourceId,
                uploadId=response.get('id')
            ).execute()
        if response.get('status') == 'FAILED':
            raise Exception(json.dumps(response.get('error'), indent=2))
        if response.get('status') == 'COMPLETED':
            pass
    
    def data_import(self, accountId, webPropertyId, dataSourceId, filename=None, content=None):
        logging.warn("method {c}.data_import is deprecated, pleas use {c}.data_import_upload_data instead.".format(c=self.__class__))
        self.data_import_upload_data(accountId, webPropertyId, dataSourceId, filename=filename, content=content)

    
    def data_import_list(self, accountId, webPropertyId, dataSourceId):

        response = self._analyticsService.management().uploads().list(
            accountId=accountId,
            webPropertyId=webPropertyId,
            customDataSourceId=dataSourceId,
            max_results=1000
        ).execute()
        imports = [item.get("id") for item in response.get("items", [])]

        while response.get("nextLink"):
            response = self._analyticsService.management().uploads().list(
                accountId=accountId,
                webPropertyId=webPropertyId,
                customDataSourceId=dataSourceId,
                max_results=1000,
                start_index= response.get("start_index") + response.get("itemsPerPage")
            ).execute()
            imports = imports + [item.get("id") for item in response.get("items", [])]
        return imports


    def data_import_delete_data(self, accountId, webPropertyId, dataSourceId, ids):
        response = self._analyticsService.management().uploads().deleteUploadData(
            accountId=accountId,
            webPropertyId=webPropertyId,
            customDataSourceId=dataSourceId,
            body={
                "customDataImportUids": ids
            }
        ).execute()

    def _in_cache_by_day(self, profile, id_, date):
        """Check if a specific report is stored in cache.

        Args:
            profile (str): profile id of the report
            id_ (str): hash id of the report
            date (str): date of the report

        Returns:
            bool: True if the report is cached, False otherwise"""
        report_path = Analytics.CACHE_UNSAMPLED_REPORT
        path = report_path.format(profile=profile, id=id_, date=date)
        return os.path.isfile(path)

    def _in_cache(self, profile, id_, start_date, end_date):
        """Check if a specific report is stored in cache.

        Args:
            profile (str): profile id of the report
            id_ (str): hash id of the report
            start_date (str): start date of the report
            end_date (boolean): end date of the report

        Returns:
            bool: True if the report is cached, False otherwise"""
        report_path = Analytics.CACHE_REPORT
        path = report_path.format(profile=profile, id=id_, start_date=start_date, end_date=end_date)
        return os.path.isfile(path)

    def clear_cache(self, id_=None, lifetime=180):
        """Clears cached reports for a given profile with in a given lifetime.

        Args:
            id_ (str): Profile id
            lifetime (int): lifetime in days, by default its 180 days
        """

        if id_ is None:
            shutil.rmtree('./cache')
            return
        today = datetime.now()
        last_day = today - timedelta(days=lifetime)
        for reports in os.listdir('./cache/{id}/'.format(id=id_)):
            for report in os.listdir('./cache/{id}/{reports}/'.format(id=id_, reports=reports)):
                day = datetime.strptime(report, 'report_%Y-%m-%d.csv')
                if day < last_day:
                    os.remove('./cache/{id}/{reports}/{report}'.format(id=id_, reports=reports, report=report))
                    logger.info('Removed cache/{id}/{reports}/{report}'.format(id=id_, reports=reports, report=report))


def filter_report_files_by_date(filename, start_date, end_date):
    """Check if a report file is within a date range.

    Args:
        filename (str): report filename
        start_date (str): start date of the report range
        end_date (str): end date of the report range

    Returns:
        bool: True if the report filename is in the date range, False otherwise"""
    start_date = datetime.strptime(start_date, "%Y-%m-%d")
    end_date = datetime.strptime(end_date, "%Y-%m-%d")
    filename_date = datetime.strptime(filename, "unsampled_report_%Y-%m-%d.csv")
    return start_date <= filename_date and end_date >= filename_date


"""BigQuery Manager module.

This module have a class with all needed functionalites
of BigQuery that AudienceScore needs.
"""
__author__ = 'Metriplica-Ayyoub&Javier'

import time
from google.cloud import bigquery
from google.api_core.exceptions import NotFound

class BigQuery(object):
    """BigQuery class.

    Manage bigQuery tables and properties.
    """

    def __init__(self, project=None, location="US"):
        """Init module initialize and create BigQuery class.

        Args:
            credentials (dcit): Credentials to access to the client services
            secrets (dict): Secrets of the Google accout to use

        Returns:
            BigQuery: with given configuration.
        """
        self.bigquery_client = bigquery.Client(project=project, location=location)

    def create_table(self, project_id, dataset_id, table_id, query, legacy=True):
        """Create table function.

        Create a table in the project_id/dataset at bigQuery.
        If the table alredy exists its gona be overwrited.

        Args:
            project_id (str): BigQuery project id
            table_id   (str): Name of the table to createTable.
            query       (str): Query to store as a table.

        Returns:
            bool: True for success, Raises an error otherwise.

        """
        table = self.bigquery_client.dataset(dataset_id, project_id).table(table_id)
        query_job_config = bigquery.QueryJobConfig()
        query_job_config.create_disposition = bigquery.CreateDisposition.CREATE_IF_NEEDED
        query_job_config.write_disposition = bigquery.WriteDisposition.WRITE_TRUNCATE
        query_job_config.destination = table
        query_job_config.use_legacy_sql = legacy
        query_job_config.allow_large_results = True
        query_job = self.bigquery_client.query(query, query_job_config)
        query_job.result()
        while query_job.running():
            time.sleep(60)
        if query_job.done():
            return True
        elif query_job.cancelled():
            raise Exception("BigQuery table creation", query_job.errors)

    def create_empty_table(self, project_id, dataset_id, table_id, schema, partition_field=None, expiration=None):
        """Create table function.

        Create a table in the project_id/dataset at bigQuery.
        If the table alredy exists its gona be overwrited.

        Args:
            project_id (str): BigQuery project id
            table_id   (str): Name of the table to createTable.
            query      (str): Query to store as a table.

        Returns:
            bool: True for success, Raises an error otherwise.

        """

        table_ref = self.bigquery_client.dataset(dataset_id, project_id).table(table_id)
        if isinstance(schema, list):
            schema = self._get_schema_from_json(schema)
        elif isinstance(schema, str):
            schema = self._get_schema_from_str(schema)
        table = bigquery.Table(table_ref, schema=schema)
        if partition_field: 
            expiration_ms = expiration*24*60*60*1000 if expiration else None
            partitioning = bigquery.TimePartitioning(field=partition_field, expiration_ms=expiration_ms)
            table.time_partitioning = partitioning
        table_ref = self.bigquery_client.create_table(table)

    @staticmethod
    def _get_schema_from_str(schema_str):
        schema = []
        for field in schema_str.split(","):
            name, type_ = field.split(":")
            schema.append(bigquery.SchemaField(name, type_))
        return schema
    
    @staticmethod
    def _get_schema_from_json(schema):
        for field in schema:
            field = bigquery.SchemaField.from_api_repr(field)
        return schema
       
    def overwrite_table(self, project_id, dataset_id, table_id, query, legacy=True):
        """Create table function.

        Create a table in the projectId/dataset at bigQuery.
        If the table alredy exists its gona be overwrited.

        Args:
            table_id   (str): Name of the table to createTable.
            query       (str): Query to store as a table.

        Returns:
            bool: True for success, Raises an error otherwise.

        """
        table = self.bigquery_client.dataset(dataset_id, project_id).table(table_id)
        query_job_config = bigquery.QueryJobConfig()
        query_job_config.create_disposition = bigquery.CreateDisposition.CREATE_NEVER
        query_job_config.write_disposition = bigquery.WriteDisposition.WRITE_TRUNCATE
        query_job_config.destination = table
        query_job_config.use_legacy_sql = legacy
        query_job_config.allow_large_results = True
        query_job = self.bigquery_client.query(query, query_job_config)
        query_job.result()
        while query_job.running():
            time.sleep(60)
        if query_job.done():
            return True
        elif query_job.cancelled():
            raise Exception("BigQuery table creation", query_job.errors)

    def append_table(self, project_id, dataset_id, table_id, query, legacy=True):
        """Append to a specified table the result of the specified query.

        Args:
            tableId   (str): Name of the table to append data.
            query       (str): Query to append to the table.

        Returns:
            bool: True for success, Raises an error otherwise.

        """
        table = self.bigquery_client.dataset(dataset_id, project_id).table(table_id)
        query_job_config = bigquery.QueryJobConfig()
        query_job_config.create_disposition = bigquery.CreateDisposition.CREATE_NEVER
        query_job_config.write_disposition = bigquery.WriteDisposition.WRITE_APPEND
        query_job_config.destination = table
        query_job_config.use_legacy_sql = legacy
        query_job_config.allow_large_results = True
        query_job = self.bigquery_client.query(query, query_job_config)
        query_job.result()
        while query_job.running():
            time.sleep(60)
        if query_job.done():
            return True
        elif query_job.cancelled():
            raise Exception("BigQuery table creation", query_job.errors)

    def delete_table(self, project_id, dataset_id, table_id):
        """Delete table function.

        Delete a table from the projectId/dataset at bigQuery.

        Args:
            tableId   (str): Name of the table to createTable.
            query       (str): Query to store as a table.

        Returns:
            bool: True for success, Raises an error otherwise.

        """

        table = self.bigquery_client.dataset(dataset_id).table(table_id)
        self.bigquery_client.delete_table(table)
        return True

    def save_query2csv(self, filename, project_id, query, header=None, delimiter=',', legacy=True):
        """Save the result of a query into a CSV.

        The CSV header are formed by de custom dimensions
        that are in the configuration of AS.

        Args:
            filename(str):  Name of the file to save.
            query   (str): Name of the query to request to BigQuery.

        Returns:
            bool: True if the query have results. False otherwise.

        """
        query_job_config = bigquery.QueryJobConfig()
        query_job_config.use_legacy_sql = legacy
        query_job = self.bigquery_client.query(query,project=project_id, job_config=query_job_config)
        query_job.result()
        while query_job.running():
            time.sleep(60)
        if query_job.done():
            data = query_job.to_dataframe()
            if header:
                data.columns = header
            data.to_csv(filename, sep=delimiter, index=False)
            return True
        elif query_job.cancelled():
            raise Exception("BigQuery table creation", query_job.errors)

    def is_table_created(self, project_id, dataset_id, table_id):
        """Check if the specified table is created.

        Args:
            tableId   (str):  Table name to Check.

        Returns:
            bool: True if exists, false otherwhise.

        """
        try:
            table_ref = self.bigquery_client.dataset(dataset_id, project_id).table(table_id)
            table = self.bigquery_client.get_table(table_ref)
            return True
        except NotFound:  # noqa
            return False

    def get_table_properties(self, project_id, dataset_id, table_id):
        """Get properties of a bigQuery table.

        Args:
            projectId (str): BigQuery project.
            datasetId (str): Dataset id of the table.
            tableId (str): Table id.
            fields (str): Requested properties

        Returs:
            dict: Json with the specified properties.

        """
        table_ref = self.bigquery_client.dataset(dataset_id, project_id).table(table_id)
        return self.bigquery_client.get_table(table_ref)
        return self._bigqueryService.tables().get(
            projectId=project_id,
            datasetId=dataset_id,
            tableId=table_id
        ).execute()

    def create_dataset(self, project_id, dataset_id, location):
        """Create a dataset in the specified project.

        Args:
            project_id (str): bigQuery project_id.
            dataset_id (str): dataset_id to create.

        Returns:
            bool: True if succesfully created. Raises error otherwhise.
        """
        #self.bigquery_client.location = location
        #parche
        try:
            dataset_ref = self.bigquery_client.dataset(dataset_id, project_id)
            dataset = self.bigquery_client.create_dataset(dataset_ref)
            return True
        except Exception:
            return True
        



from setuptools import setup
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

setup(
    name='pykemen',
    version='0.2.11',
    description='Added data import management features',
    license='Eiup',
    packages=find_packages(),
    include_package_data=True,
    package_data={'pykemen': ['google/*.py', '*']},
    author='Ayyoub Marroun',
    author_email='ayyoub@metriplica.com',
    keywords=['google apis', 'utilities'],
    url='https://github.com/ayyoubmarroun/pykemen',
    download_url='https://github.com/ayyoubmarroun/pykemen/archive/0.2.11.tar.gz',
    install_requires=[
        'httplib2',
        'pandas',
        'google-api-python-client',
        'oauth2client',
        'google-cloud',
        'google-cloud-core',
        'google-cloud-bigquery',
        'future'
    ]
)


"""Mail Manager module.

This module have a class with all needed functionalites
of Mail that AudienceScore needs.
"""
__author__ = 'Metriplica-Ayyoub&Javier'

import base64
import pykemen.utilities as utilities
from email.mime.text import MIMEText
from apiclient import errors  # noqa


class Mail(object):
    """Mail class.

    Useful to send messages form the specified account.
    """

    def __init__(self, secrets, credentials):
        """Init module initialize and create Mail class.

        Args:
            credentials (str): Credentials to access to the client services
            secrets (str): Secrets of the Google accout to use

        Returns:
            Mail: with given configuration.
        """
        scopes = scopes = [
            "https://www.googleapis.com/auth/gmail.send", 
            "https://www.googleapis.com/auth/bigquery", 
            "https://www.googleapis.com/auth/bigquery.insertdata",
            ]
        self._gmailService = utilities.create_api('gmail', 'v1', scopes, secrets, credentials)

    def _createMessage(self, to, subject, body, type='plain'):
        """Create a message object."""
        message = MIMEText(body, type)
        message['to'] = ','.join(to)
        message['from'] = 'me'
        message['subject'] = subject
        return {'raw': base64.urlsafe_b64encode(message.as_string())}

    def sendMessage(self, to, subject, message, type='plain'):
        """Send a message to specified reciver.

        Args:
            to (list): Recivers of the email separated by comma
            subject (str): subject of the email
            message (str): message or body of the email

        Returns:
            int: message id.

        """
        try:
            objectMessage = self._createMessage(to, subject, message, type)
            messageId = self._gmailService.users().messages().send(
                userId='me', body=objectMessage).execute()
            return messageId
        except errors.HttpError as error:
            raise Exception(error)
